
/*!
 * jade-serial - Serializer
 * Copyright (c) 2011 LearnBoost <dev@learnboost.com>
 * MIT Licensed
 */

/**
 * Module dependencies.
 */

var jade = require('jade');

/**
 * Expose Serializer.
 */

module.exports = Serializer;

/**
 * Initialize Serializer with the given node.
 */

function Serializer(node) {
  this.node = node;
}

Serializer.prototype = {
  
  /**
   * Serialize the AST, returning a JSON friendly object.
   */
  
  serialize: function(){
    return this.visit(this.node);
  },
  
  /**
   * Visit `node`.
   */
  
  visit: function(node){
    if (Array.isArray(node)) return node.map(this.visit.bind(this));
    var type = node.constructor.name.toLowerCase()
      , obj = { __type: type };
    if ('object' != typeof node) return node;
    Object.keys(node).forEach(function(key){
      obj[key] = node[key];
    });
    if (node.nodes) obj.__items = this.visit(node.nodes);
    if (node.block) obj.block = this.visit(node.block);
    if (node.text) obj.text = this.visit(node.text);
    delete obj.line;
    delete obj.textOnly;
    delete obj.nodes;
    return obj;
  }
};